@@ -7,8 +7,10 @@ from concurrent.futures import ThreadPoolExecutor
 import pandas as pd
 from rich.progress import Progress
 
+from .cas import CAS
 from .config import settings
 from .db import get_DB_client
+from .models.objects import ObjResult
 from .save import get_save_client
 
 logger = logging.getLogger(__name__)
@@ -19,6 +21,11 @@ class Collect:
         self.sh = get_save_client(save)
         self.db = get_DB_client(settings.dbm)
 
+        if settings.get("CAS", None):
+            self.cas = CAS(save=save, **settings.cas)
+        else:
+            self.cas = None
+
     async def run(self, classes):
         if isinstance(classes, str):
             classes = [row for row in self.get_rows(level="CLASSES", sql=classes)]
@@ -68,15 +75,28 @@ class Collect:
         if self.sh.check_obj(cdclass, key):
             logger.info(f"Object {key} already exists")
             return True
-        for key, part, obj in self.db.get_objects(key):
-            try:
-                if isinstance(obj, bytes):
-                    obj = io.BytesIO(obj)
-                self.sh.save_obj(obj.read(), cdclass, key, part)
-            except AttributeError:
-                logger.error(f"The key {key} has a empty file, it will be ignored")
+        for row in self.db.get_objects(key):
+            result = ObjResult(*row)
+            if result.idcas and self.cas:
+                logger.info(
+                    f"CAS: id={result.idcas.strip()}, {cdclass}, {result.key}, {result.part}"
+                )
+                # loop = asyncio.get_event_loop()
+                # result.obj = loop.run_until_complete(
+                #     self.cas.retrieve_obj(result.idcas)
+                # )
+            if result.obj:
+                self.sh.save_obj(
+                    io.BytesIO(result.obj).read(), cdclass, result.key, result.part
+                )
+            else:
+                logger.error(
+                    f"The key {result.key} has a empty file in part {result.part}, it will be ignored"
+                )
 
-    def get_rows(self, cdclass=None, level="DOCS", sql=settings.keys_sql):
+    def get_rows(self, cdclass=None, level="DOCS", sql=None):
+        if not sql:
+            sql = settings.keys_sql
         if not cdclass:
             cdclass = sql
         exists, docs = self.sh.check_meta(f"{level}_LIST_{cdclass}")
